<?php


class views_content_ds_FieldCollector {

  /**
   * @var array[]
   *   Fields configuration by entity type and field name. E.g.
   *
   *   $this->fields['node']['field_body'] = array(
   *
   *   );
   */
  protected $fields = array();

  /**
   * @var string[]
   */
  protected $entityTypesMap = array();

  /**
   * @param string $typeName
   * @param string $primaryKeyName
   *   Name of the entity type's primary key.
   */
  function addEntityType($typeName, $primaryKeyName) {
    $this->entityTypesMap["entity:$typeName.$primaryKeyName"] = $typeName;
  }

  /**
   * Get the collected result.
   *
   * @return array[]
   *   The collected fields information.
   */
  function getCollectedFields() {
    return $this->fields;
  }

  /**
   * Register a content_pane views display, with argument options.
   *
   * @param \views_display $display
   * @param array $arg_options
   */
  function registerViewsDisplay(\views_display $display, array $arg_options) {
    $key = str_replace('-', '_', $arg_options['context']);
    if (isset($this->entityTypesMap[$key])) {
      $this->_viewsPane($this->entityTypesMap[$key], $display);
    }
    elseif ($key === 'entity:group.entity_id') {
      $this->_viewsPane('node', $display);
    }
  }

  /**
   * @param string $entity_type
   * @param \views_display $display
   */
  protected function _viewsPane($entity_type, $display) {
    $view_id = $display->handler->view->name;
    $fieldname = 'views_content_ds__' . $view_id . '__' . $display->id;
    $fieldname_nice = t($display->handler->view->human_name);
    $fieldname .= '__' . $display->id;
    $fieldname_nice .= ': ' . t($display->display_title);
    $admin_path = "admin/structure/views/view/$view_id/edit/$display->id";
    $title = t('!field_name (Views content DS)', array(
      '!field_name' => $fieldname_nice,
    ));
    $this->fields[$entity_type][$fieldname] = array(
      'title' => $title,
      'field_type' => DS_FIELD_TYPE_FUNCTION,
      'function' => '_views_content_ds_field_print',
      // These two settings have no meaning for ds api,
      // we just need them in the callback.
      '_view_id' => $view_id,
      '_display_id' => $display->id,
      // We could restrict to specific bundles and view modes, if we wanted.
      'ui_limit' => array('*|*'),
    );
  }

  /**
   * @param string $entity_type
   * @param string $fieldname
   * @param \views_display|object $display
   */
  protected function _formatter($entity_type, $fieldname, $display) {
    $formatter_name = $display->id;
    $formatter_title = t($display->display_title);
    $this->fields[$entity_type][$fieldname]['properties']['formatters'][$formatter_name] = $formatter_title;
  }
}
